dojo.provide("zero.form.widget.FormController");

dojo.require("zero.resource.DataStore");

dojo.declare ("zero.form.widget.FormController", null, {

    contextRoot: "",
    resourceCollection: "",
    store: null,
    
    isStoreReady: false,
    
    constructor: function(/* Object */ controllerParameters)
	{
        // get parameters
	    this.resourceCollection = controllerParameters.resourceCollection;
        this.contextRoot = controllerParameters.contextRoot;
        this.store = controllerParameters.store;
        
        // user given store or create a default one
        if(this._isValid())
		{
            if(!this.store)
			{// create a default ZRM store if external store is not specified
                	this.store = new zero.resource.DataStore({contextRoot:this.contextRoot, resourceCollection:this.resourceCollection});
            } 
			else 
			{
                if (this.resourceCollection.length > 0 || this.contextRoot.length > 0) 
				{
                    console.debug("Ignoring resourceCollection or contextRoot attribute set in controller in favor of those specified in store:"
                        + this.store);
                }
            }
        } 
		else 
		{
            throw new Error("Neither resourceCollection nor contextRoot is defined");
        } 
       
       // the store does not work without an initial fetch 
       // TODO Fix this - maybe already a fetch was done, but there is no method 
       // to check that in the store, they check it with this condition....
       //Only because ZRM data store need a fetch before work
       if((!this.store._baseZRMCollectionURI)){
	       	this.store.fetch({
	           onComplete: dojo.hitch(this,this.onReady), 
	           onError: dojo.hitch(this,this.onError)});	
	   }else{
	   		this.onReady();
	   }
    },
    
	// assumption - the received store was already fetched
	setStore: function(/*zero.resource.DataStore */store)
	{
		store && (this.store = store);
	},
	
	getStore: function(){
		return this.store;
	},
	
    _isValid: function()
	{
        var valid = true;
        if( !this.store && (this.resourceCollection === null || this.resourceCollection.length == 0)){
            console.debug("Missing required attribute, resourceCollection (ex. 'employees')");
            valid = false;
        }
        return valid;
    },
    
    getItemById: function(itemId,/* Function */ onItem, /* Function */ onError){
    	//ZRM data store is little bit weird in the item identity, e.g. "100_"
    	this.store.fetchItemByIdentity({
    		identity: itemId,
    		"onItem": onItem,
    		"onError":onError
    	});
    },
  
    addItem: function (/* Object? */ keywordArgs) 
	{
        var newItem = this.store.newItem(keywordArgs);
        this.store.save({
                onComplete: dojo.hitch(this,this.onSaved, newItem), 
                onError: dojo.hitch(this,this.onError)});
        return (newItem);
    },
    
    /* Store item is a reference to the item from the store to be updated  */  
    updateItem: function (/*Object */storeItem, /* Object? */ keywordArgs) 
	{
        // update store item 
        for (var attribute in keywordArgs) 
		{
            this.store.setValue(storeItem, attribute, keywordArgs[attribute]);            
        }
        // save changes in store
        this.store.save({
                onComplete: dojo.hitch(this,this.onSaved, storeItem), 
                onError: dojo.hitch(this,this.onError)});
    },
    
    // Store updated 
    onSaved: function(/*Object*/ item)
	{  
        console.log('Store Updated with item '+item);
    },
    
    
    onReady: function()
	{    
        console.log('Controller is ready for usage.');
        this.isStoreReady = true;
    },
        
    onError: function(errorData)
	{
        this.isStoreReady = false;
        console.log('Error: ' + errorData);
        
    }
    
});